/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ercp.eworkbench;

import java.util.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IPluginContribution;
import org.eclipse.ui.internal.IWorkbenchGraphicConstants;
import org.eclipse.ui.internal.WorkbenchImages;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.internal.registry.RegistryReader;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;


/**
 * class used to describe an eRCP application
 */
public class ApplicationDescriptor implements IApplicationDescriptor,
        IPluginContribution {

	/**
	 * @uml.property  name="id"
	 */
	private String id;
    /**
	 * @uml.property  name="pluginId"
	 */
    private String pluginId;
    /**
	 * @uml.property  name="label"
	 */
    private String label;
    /**
	 * @uml.property  name="className"
	 */
    private String className;
    /**
	 * @uml.property  name="description"
	 */
    private String description;
    /**
	 * @uml.property  name="singleton"
	 */
    private boolean singleton;
    private ImageDescriptor image; 
    /**
	 * @uml.property  name="configElement"
	 */
    private IConfigurationElement configElement;
	private Hashtable views = new Hashtable();
	

    /**
     * Create a new empty descriptor.
     * 
     * @param id the id of the new descriptor
     * @param label the label of the new descriptor
     * @param originalDescriptor the descriptor that this descriptor is based on
     */
    public ApplicationDescriptor(String id, String label) {
        this.id = id;
        this.label = label;
    }

    /**
     * Create a descriptor from a config element.
     * 
     * @param id the id of the element to create
     * @param configElement the element to base this perspective on
     * @throws CoreException thrown if there are any missing attributes
     */
    public ApplicationDescriptor(String id, IConfigurationElement configElement) throws CoreException {
        this.configElement = configElement;
        this.id = id;
        // Sanity check.
        if ((getId() == null) || (getLabel() == null) ) {
            throw new CoreException(new Status(IStatus.ERROR,
                    WorkbenchPlugin.PI_WORKBENCH, 0,
                    "Invalid extension (missing label, id or class name): " + getId(),//$NON-NLS-1$
                    null));
        }
    }
    
    /**
	 * @return
	 * @uml.property  name="description"
	 */
    public String getDescription() {
        return configElement == null ? description : RegistryReader.getDescription(configElement);
    }

    /**
	 * @return
	 * @uml.property  name="id"
	 */
    public String getId() {
        return id;
    }

    public ImageDescriptor getImageDescriptor() {
    	if (image == null) {
            if (configElement != null) {
                String icon = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ICON);
                if (icon != null) {
                    image = AbstractUIPlugin.imageDescriptorFromPlugin(
                            configElement.getNamespace(), icon);
                }
                if (image == null) {
                    image = WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_ETOOL_DEF_PERSPECTIVE);
                }
            }
    	}
        return image;
    }

    /**
	 * @return
	 * @uml.property  name="label"
	 */
    public String getLabel() {
        return configElement == null ? label : configElement.getAttribute(IWorkbenchRegistryConstants.ATT_NAME);
    }

    /**
	 * Returns <code>true</code> if this perspective is a singleton.
	 * @return  whether this perspective is a singleton
	 * @uml.property  name="singleton"
	 */
    public boolean isSingleton() {
        return configElement == null ? singleton : configElement.getAttributeAsIs(IWorkbenchRegistryConstants.ATT_SINGLETON) != null;
    }

    /**
	 * Return the configuration element used to create this perspective, if one was used.
	 * @return  the configuration element used to create this perspective
	 * @uml.property  name="configElement"
	 */
    public IConfigurationElement getConfigElement() {
        return configElement;
    }

    public String getLocalId() {
        return getId();
    }

    /**
	 * @return
	 * @uml.property  name="pluginId"
	 */
    public String getPluginId() {
        return configElement == null ? pluginId : configElement.getNamespace();
    }
    
    /**
	 * Returns the factory class name for this descriptor.
	 * @return  the factory class name for this descriptor
	 * @uml.property  name="className"
	 */
    public String getClassName() {
    	return configElement == null ? className : RegistryReader.getClassValue(configElement, IWorkbenchRegistryConstants.ATT_CLASS);
    }
	
	public void setViewId(String type, String id) {
		views.put(type, id);
	}
	
	public String getViewId(String type) {
		return (String)views.get(type);
	}
}
