/*
 * Copyright 2009 UbiCollab.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ubicollab.core.sdm.model;

import java.util.*;
import org.osgi.framework.*;


/**
 * The Class <code>Activator</code> contains the start and stop members, which is used
 * by the framework to start/activate and stop/deactivate this bundle (and thus the 
 * Service Domain Manager).
 * <p>
 * This bundle provides management functionality for services/bundles and will provide
 * a SOAP/Web-Service interface.
 * 
 * @author Kim-Steve Johansen
 * @version 0.2
 * @since 0.1
 */
public class Activator implements BundleActivator {
	
	/** Handle to the Bundle Context. */
	private BundleContext bc = null;
	
	/** The Service Domain Manager implementation, implements <code>DomainManager</code>. */
	private DomainManagerImpl domainManager = null;
	
	
	
	/**
	 * This method is invoked by the framework to start the bundle and its service 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			this.bc = bc;

			domainManager = new DomainManagerImpl(this.bc);
			
			// external 
			Properties propertiesExternal = new Properties();
			propertiesExternal.put("Language", "English");
			propertiesExternal.put("SOAP.service.name", GlobalSettings.SOAP_SERVICE_NAME);
			bc.registerService(DomainManager.class.getName(), domainManager, propertiesExternal);

			// internal
			Properties propertiesInternal = new Properties();
			propertiesInternal.put("Language", "English");
			bc.registerService(DomainManagerInternal.class.getName(), domainManager, propertiesInternal);
			
			log.writeToLog("Service domain manager has started.", Log.INFO);
			System.out.println("Service Domain Manager has started.");
		} catch (Exception e) {
			log.writeToLog("Failed to start Service Domain Manager.", Log.ERROR);
			throw new BundleException("Failed to start Service Domain Manager", e);
		}
	}
	
	
	
	/**
	 * This method is invoked by the framework to stop the bundle and its service.
	 * All resources used is released to the garbage collector. 
	 * 
	 * @param bc A refrence to the Frameworks BundleContext 
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bc) throws BundleException {
		Log log = Log.getInstance();
		try {
			bc = null;
			domainManager = null;
			log.writeToLog("Service Domain Manager has stopped.", Log.INFO);
			System.out.println("Service Domain Manager has stopped.");
		} catch (Exception e) {
			log.writeToLog("Failed to stop Service Domain Manager.", Log.ERROR);
			throw new BundleException("Failed to stop Service Domain Manager", e);
		}
	}
	
	public static void main(String[] args) {
		DomainManagerImpl i = new DomainManagerImpl(null);
		DomainManagerInternal internal = (DomainManagerInternal) i;
		DomainManager external = (DomainManager) i;
		System.out.println(external + ", " + internal);
	}

}