/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.image;

import java.awt.image.ColorModel;
import java.awt.image.ImageFilter;
import java.util.Arrays;

/**
 * Image filter to emulate the spacing between leds on the wall.
 * If used in combination (after) the RGB2LedImageFilter,
 * a wall-like image may be produced.
 * @author Hallvard Trtteberg
 *
 */
public class WallImageFilter extends ImageFilter {

	private int scale = 3;
	private int backgroundPixel = 0xff000000;
	
	private int[] iRowPixels = null;
	private int[] iBackgroundPixels = null;
	
	private byte[] bRowPixels = null;
	private byte[] bBackgroundPixels = null;
	
	public void setDimensions(int width, int height) {
		int scaledWidth = width * scale;
		super.setDimensions(scaledWidth, height * scale);
		iRowPixels = new int[scaledWidth];
		iBackgroundPixels = new int[scaledWidth];
    	Arrays.fill(iBackgroundPixels, backgroundPixel);
		bRowPixels = new byte[scaledWidth];
		bBackgroundPixels = new byte[scaledWidth];
		Arrays.fill(bBackgroundPixels, (byte)(backgroundPixel & 0xff));
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int off, int scansize) {
	    int index = off;
	    for (int dy = 0; dy < h; dy++) {
			for (int d = 0; d < scale; d++) {
				super.setPixels(x * 3, y * 3 + d, w * 3, 1, model, bBackgroundPixels, 0, bBackgroundPixels.length);
	    	}
	    	System.arraycopy(bBackgroundPixels, 0, bRowPixels, 0, bRowPixels.length);
	    	int scaleHalf = scale / 2;
			for (int dx = 0; dx < w; dx++) {
				int rgb = pixels[index++];
				bRowPixels[dx * 3 + scaleHalf + 1] = (byte)rgb;
			}
			super.setPixels(x * 3, y * 3 + scaleHalf + 1, w * 3, 1, model, bRowPixels, 0, bRowPixels.length);
			index += scansize - w;
	    }
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int off, int scansize) {
	    int index = off;
	    for (int dy = 0; dy < h; dy++) {
			for (int d = 0; d < scale; d++) {
	    		super.setPixels(x * 3, y * 3 + d, w * 3, 1, model, iBackgroundPixels, 0, iBackgroundPixels.length);
	    	}
	    	System.arraycopy(iBackgroundPixels, 0, iRowPixels, 0, iRowPixels.length);
	    	int scaleHalf = scale / 2;
			for (int dx = 0; dx < w; dx++) {
				int rgb = pixels[index++];
				iRowPixels[dx * 3 + scaleHalf + 1] = rgb;
			}
			super.setPixels(x * 3, y * 3 + scaleHalf + 1, w * 3, 1, model, iRowPixels, 0, iRowPixels.length);
			index += scansize - w;
	    }
	}
}
