/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.image;

import java.awt.image.ColorModel;
import java.awt.image.ImageConsumer;
import java.util.Hashtable;

import no.ntnu.idi.wall.model.WallModel;

/**
 * Image consumer that updates a WallModel, based on the red component of each pixel. Flipping, 90-degree rotations and translation may be performed (in that order).
 * @author  Hallvard Trtteberg
 */
public class WallModelImageConsumer implements ImageConsumer {

	/**
	 * @uml.property  name="model"
	 * @uml.associationEnd  
	 */
	private WallModel model;
	private int offsetX = 0;
	private int offsetY = 0;

	public WallModelImageConsumer(WallModel model) {
		this.model = model;
	}
	
	/**
	 * Sets the translation.
	 * @param x translation in x-direction
	 * @param y translation in y-direction
	 */
	public void setOffset(int x, int y) {
		offsetX = x;
		offsetY = y;
	}
	
	private boolean flipHorizontally = false;
	private boolean flipVertically   = false;

	/**
	 * Controls the horizontal flip (mirror)
	 * @param  flipHorizontally
	 * @uml.property  name="flipHorizontally"
	 */
	public void setFlipHorizontally(boolean flipHorizontally) {
		this.flipHorizontally = flipHorizontally;
	}
	
	/**
	 * Controls the vertical flip (mirror)
	 * @param  flipVertically
	 * @uml.property  name="flipVertically"
	 */
	public void setFlipVertically(boolean flipVertically) {
		this.flipVertically = flipVertically;
	}
	
	private int rotateClockwise = 0;
	private int rotateCounterClockwise = 0;
	
	/**
	 * Number of 90-degree clockwise rotations to perform.
	 * @param  rotateClockwise
	 * @uml.property  name="rotateClockwise"
	 */
	public void setRotateClockwise(int rotateClockwise) {
		this.rotateClockwise = rotateClockwise;
	}

	/**
	 * Number of 90-degree counter-clockwise rotations to perform.
	 * @param  rotateCounterClockwise
	 * @uml.property  name="rotateCounterClockwise"
	 */
	public void setRotateCounterClockwise(int rotateCounterClockwise) {
		this.rotateCounterClockwise = rotateCounterClockwise;
	}

	private void setLed(int x, int y, int argb) {
		int modelX = (flipHorizontally ? width  - x - 1 : x);
		int modelY = (flipVertically   ? height - y - 1 : y);
		int r = rotateClockwise - rotateCounterClockwise;
		while (r < 0) {
			r += 4;
		}
		while (r-- > 0) {
			int temp = modelX;
			modelX = width - modelY;
			modelY = temp;
		}
		modelX += offsetX;
		modelY += offsetY;
		
		int alpha = 255; // (argb >> 24) & 0xff;
		int newRed = (argb >> 16) & 0xff;
		int oldRed = this.model.getLed(modelX, modelY) * 255 / WallModel.MAX_LED_VALUE;
		int red = (oldRed * (255 - alpha) + newRed * alpha) / 255;
		this.model.setLed(modelX, modelY, (byte)(red * WallModel.MAX_LED_VALUE / 255));
	}
	
	public void setPixels(int x, int y, int w, int h, ColorModel model, byte[] pixels, int off, int scansize) {
		int index = off;
		for (int dy = 0; dy < h; dy++) {
			for (int dx = 0; dx < w; dx++) {
				setLed(x + dx, y + dy, model.getRGB(pixels[index + dx] & 0xff));
			}
			index += scansize;
		}
	}

	public void setPixels(int x, int y, int w, int h, ColorModel model, int[] pixels, int off, int scansize) {
		int index = off;
		for (int dy = 0; dy < h; dy++) {
			for (int dx = 0; dx < w; dx++) {
				setLed(x + dx, y + dy, model.getRGB(pixels[index + dx]));
			}
			index += scansize;
		}
	}

	private int width;
	private int height;
	
	public void setDimensions(int width, int height) {
		this.width = width;
		this.height = height;
	}

	public void imageComplete(int status) {
	}
	public void setColorModel(ColorModel model) {
	}

	public void setHints(int hintflags) {
	}
	public void setProperties(Hashtable<?, ?> props) {
	}
}
