/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.model;

/**
 * A default implementation of WallModel, based on AbstractWallModel.
 * @author Hallvard Trtteberg
 *
 */
public class DefaultWallModel extends AbstractWallModel {

	private byte[][] leds = new byte[WIDTH][HEIGHT];
	private boolean[][] dirty = null;
	
	public DefaultWallModel() {
	}
	
	/**
	 * Determines if notifications are sent lazily, i.e. delayed and aggregated,
	 * based on the dirty state of panels.
	 * @return whether or not notifications are sent lazily
	 */
	public boolean isLazyNotification() {
		return dirty != null;
	}
	
	/**
	 * Sets whether or not notifications are sent lazily, i.e. delayed and aggregated,
	 * based on the dirty state of panels.
	 * @param on the value
	 */
	public void setLazyNotification(boolean on) {
		if (on != isLazyNotification()) {
			if (on) {
				dirty = new boolean[WIDTH / NODE_WIDTH][HEIGHT / NODE_HEIGHT];
			} else {
				notifyListeners();
				dirty = null;
			}
		}
	}
	
	/**
	 * Notifies all listeners about changes to the leds,
	 * based on the dirty state. Notifications are aggregated for each panel,
	 * to minimize network traffic.
	 */
	public void notifyListeners() {
		if (dirty != null) {
			for (int i = 0; i < dirty.length; i++) {
				for (int j = 0; j < dirty[i].length; j++) {
					if (dirty[i][j]) {
						super.notifyListeners(i * NODE_WIDTH, j * NODE_HEIGHT, NODE_WIDTH, NODE_HEIGHT, (byte)-1);
						dirty[i][j] = false;
					}
				}
			}
		}
	}
	
	public byte getLed(int x, int y) {
		return leds[x][y];
	}

	/**
	 * Overrides the default method, to support the dirty state flags.
	 */
	protected void notifyListeners(int x, int y, int width, int height, byte value) {
		if (isLazyNotification()) {
			for (int wx = x / WallModel.NODE_WIDTH; wx * WallModel.NODE_WIDTH < x + width; wx++) {
				for (int wy = y / WallModel.NODE_HEIGHT; wy * WallModel.NODE_HEIGHT < y + height; wy++) {
					dirty[wx][wy] = true;
				}
			}
		} else {
			super.notifyListeners(x, y, width, height, value);
		}
	}

	public void setLed(int x, int y, byte value) {
		if (value != leds[x][y]) {
			leds[x][y] = value;
			notifyListeners(x, y, 1, 1, value);
		}
	}
	
	public void setLeds(int x0, int y0, int dx, int dy, byte value) {
		for (int x = x0; x < x0 + dx; x++) {
			for (int y = y0; y < y0 + dy; y++) {
				leds[x][y] = value;
			}
		}
		notifyListeners(x0, y0, dx, dy, value);
	}
	
	/**
	 * Clears (set the value to 0) all leds.
	 */
	public void clearLeds() {
		setLeds(0, 0, WIDTH, HEIGHT, (byte)0);
	}
}
