/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.model;

/**
 * This is the logical model for the wall: A two-dimensional array of led values.
 * @author Hallvard Trtteberg
 *
 */

public interface WallModel {

	/**
	 * The maximum led value
	 */
	public static final int MAX_LED_VALUE = 99;

	/**
	 * The number of leds on each panel, in the x-direction.
	 */
	public final static int NODE_WIDTH = 5;

	/**
	 * The number of leds on each panel, in the y-direction.
	 */
	public final static int NODE_HEIGHT = 5;
	
	/**
	 * The number of leds on the wall, in the x-direction.
	 */
	public static final int WIDTH = 16 * NODE_WIDTH;

	/**
	 * The number of leds on the wall, in the y-direction.
	 */
	public static final int HEIGHT = 6 * NODE_HEIGHT;

	/**
	 * Adds a WallListener to the set of listeners to the model.
	 * @param listener
	 */
	public void addWallListener(WallListener listener);

	/**
	 * Removes a WallListener from the set of listeners to the model.
	 * @param listener
	 */
	public void removeWallListener(WallListener listener);

	/**
	 * Returns the value of the led at position x,y, where 0,0 represents the upper left corner of the wall.
	 * @param x the x-coordinate, 0 is left, WIDTH-1 is right. 
	 * @param y the y-coordinate, 0 is top, HEIGHT-1 is bottom.
	 * @return the value of the led at position x,y
	 */
	public byte getLed(int x, int y);

	/**
	 * Sets the value of the led at position x,y, to value.
	 * @param x the x-coordinate, 0 is left, WIDTH-1 is right. 
	 * @param y the y-coordinate, 0 is top, HEIGHT-1 is bottom.
	 * @param value the new led value, between 0 and MAX_LED_VALUE
	 */
	public void setLed(int x, int y, byte value);

	/**
	 * Sets the value of the leds in a rectangle to value.
	 * @param x the x-coordinate, 0 is left, WIDTH-1 is right. 
	 * @param y the y-coordinate, 0 is top, HEIGHT-1 is bottom.
	 * @param width width of the rectangle to fill. 
	 * @param height height of the rectangle to fill.
	 * @param value the new led value, between 0 and MAX_LED_VALUE
	 */
	public void setLeds(int x, int y, int width, int height, byte value);
}
