/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.model;

import java.io.IOException;
import java.net.UnknownHostException;

import no.ntnu.idi.wall.server.ProtocolClient;

/**
 * A WallListener that relays change notifications to a wall server, using the server Protocol.
 * @author  Hallvard Trtteberg
 */

public class WallModelProtocolClient extends ProtocolClient implements WallListener {

	/**
	 * @uml.property  name="model"
	 * @uml.associationEnd  
	 */
	private WallModel model;
	/**
	 * @uml.property  name="dx"
	 */
	private int dx = 0;
	/**
	 * @uml.property  name="dy"
	 */
	private int dy = 0;
	
	/**
	 * This instance will send changes to a WallModel to the given serverhost.
	 * @param serverHost the serverhost to send communicate with
	 * @param model the WallModel to listen to
	 * @param dx offset in the x-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall to the right
	 * @param dy offset in the y-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall down
	 * @throws UnknownHostException
	 */
	public WallModelProtocolClient(String serverHost, WallModel model, int dx, int dy) throws UnknownHostException {
		super(serverHost);
		this.model = model;
		this.dx = dx;
		this.dy = dy;
		clearWall();
		try {
			send();
		} catch (IOException e) {
			System.err.println(e);
		}
		setModel(model);
	}

	/**
	 * This instance will send changes to a WallModel to the default wallidi.idi.ntnu.no serverhost.
	 * @param model the WallModel to listen to
	 * @param dx offset in the x-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall to the right
	 * @param dy offset in the y-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall down
	 * @throws UnknownHostException
	 */
	public WallModelProtocolClient(WallModel model, int dx, int dy) throws UnknownHostException {
		this("wallidi.idi.ntnu.no", model, dx, dy);
	}

	/**
	 * Gets the offset in the x-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall to the right.
	 * @return  the offset in the x-coordinate
	 * @uml.property  name="dx"
	 */
	public int getDx() {
		return dx;
	}

	/**
	 * Sets the offset in the x-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall to the right.
	 * @param dx  the offset in the x-coordinate
	 * @uml.property  name="dx"
	 */
	public void setDx(int dx) {
		this.dx = dx;
		updateAll(model);
	}
	
	/**
	 * Gets the offset in the y-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall own.
	 * @return  the offset in the y-coordinate
	 * @uml.property  name="dy"
	 */
	public int getDy() {
		return dy;
	}

	/**
	 * Sets the offset in the y-coordinate of the led on the wall, a positive value shifts the point on the corresponding wall down.
	 * @param dy  the offset in the y-coordinate
	 * @uml.property  name="dy"
	 */
	public void setDy(int dy) {
		this.dy = dy;
		updateAll(model);
	}

	/**
	 * Sets the WallModel that this WallListener is attached to.
	 * @param  model
	 * @uml.property  name="model"
	 */
	public void setModel(WallModel model) {
		if (this.model != null) {
			this.model.removeWallListener(this);
		}
		this.model = model;
		if (model != null) {
			model.addWallListener(this);
			updateAll(model);
		}
	}

	/**
	 * The mapping function, defaults to the identity.
	 * @param value the value to map
	 * @return the value to use for the led
	 */
	protected byte value2Led(byte value) {
		return value;
	}
	
	private short[] values = new short[WallModel.WIDTH * WallModel.HEIGHT];
	
	/**
	 * Updates the wall to correspond to the WallModel
	 * @param model the WallModel
	 */
	public void updateAll(WallModel model) {
		clearWall();
		updateWH(0, 0, WallModel.WIDTH, WallModel.HEIGHT, model);
	}
	
	/**
	 * Updates a rectangle of the wall to correspond to the WallModel
	 * @param x the x-coordinate of the rectangle
	 * @param y the y-coordinate of the rectangle
	 * @param w the width of the rectangle
	 * @param h the height of the rectangle
	 * @param model the WallModel
	 */
	public void updateWH(int x, int y, int w, int h, WallModel model) {
		if (model == null) {
			model = this.model;
		}
		if (model == null) {
			return;
		}
		for (int dy = 0; dy < h; dy++) {
			for (int dx = 0; dx < w; dx++) {
				values[(dy  + this.dy) * w + (dx + this.dx)] = (short)value2Led(model.getLed(x + dx, y + dy));
			}
		}
		setLeds((short)(x + dx), (short)(y + dy), (short)w, (short)h, values, 0);
		try {
			send();
		} catch (IOException e) {
			System.err.println(e);
		}
	}

	public void ledChanged(int x, int y, byte value, WallModel model) {
		short led = (short)value2Led(model.getLed(x, y));
		setLed((short)(x + dx), (short)(y + dy), led);
		try {
			send();
		} catch (IOException e) {
			System.err.println(e);
		}
	}

	public void ledsChanged(int x, int y, int width, int height, WallModel model) {
		updateWH(x, y, width, height, model);
	}
}
