/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.server;

/**
 * This interface defines the network protocol for the wall.
 * Each operation is defined by an opcode, declared as constants below,
 * and a set of parameters.
 * @author Hallvard Trtteberg
 *
 */
public interface Protocol {

	// opcodes
	public byte
	
		// does nothing (and ensure that zeros are ignored)
		NOP = 0
		;

	public byte SYNC = 1;

	public byte CLEAR = SYNC + 1;

	public byte SET_LED = CLEAR + 1 // byte x, byte y, byte value  
;

	public byte FILL_RECT = SET_LED + 1 // byte x, byte y, byte w, byte h, byte value
;

	public byte SET_LEDS = FILL_RECT + 1 // byte x, byte y, byte w, byte h, byte[w x h] value
;

	public byte GET_LEDS = SET_LEDS + 1 // byte x, byte y, byte w, byte h, byte[4] ip, short p -
;
	
	public String[] OPCODES = {
		"nop", "sync", "clearWall", "setLed", "fillRect", "setLeds", "getLeds",	
	};
	
	/**
	 * Does nothing. The opcode is zero, to ensure zeros are interpreted as nops. 
	 */
	public void nop();
	/**
	 * Immediately sends the currently buffered operations
	 */
	public void sync();

	/**
	 * Clears the entire wall
	 */
	public void clearWall();
	
	/**
	 * Sets the led intesity value at x,y to value
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param value the new led value
	 */
	public void setLed(short x, short y, short value);
	
	/**
	 * Fills the rectangle at x,y with dimensions w,h with value
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param w the width
	 * @param h the height
	 * @param value the new led value
	 */
	public void fillRect(short x, short y, short w, short h, short value);
	
	/**
	 * Fills the rectangle at x,y with dimensions w,h to value(s)
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param w the width
	 * @param h the height
	 * @param value the new led values, an array of at least offset + w * h shorts
	 * @param offset the offset into the value array where the values are
	 */
	public void setLeds(short x, short y, short w, short h, short[] value, int offset);
	
	/**
	 * Sends the values of the rectangle at x,y with dimensions w,h to ip at port p
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param w the width
	 * @param h the height
	 * @param ip the host to send the values to, as a short[4] array
	 * @param port the port to send the values to
	 */
	public void getLeds(short x, short y, short w, short h, short[] ip, int port);
}
