/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

/**
 * An implementation of Protocol that serializes each operation and sends them to a server. The serialization is the opcode followed by the parameters in order.
 * @author  Hallvard Trtteberg
 */
public class ProtocolClient implements Protocol {

	private InetAddress serverHost;
	private int serverPort;
	
	/**
	 * This ProtocolClient sends the operations to serverHost and serverPort.
	 * @param serverHost
	 * @param serverPort
	 */
	public ProtocolClient(InetAddress serverHost, int serverPort) {
		this.serverHost = serverHost;
		this.serverPort = serverPort;
	}
	
	/**
	 * Send the clearWall operation.
	 */
	protected void sendClearWall() {
		clearWall();
		try {
			send();
		} catch (IOException e) {
		}
	}

	/**
	 * This ProtocolClient sends the operations to serverHost and ProtocolServer.SERVER_PORT.
	 * @param serverHost
	 */
	public ProtocolClient(InetAddress serverHost) {
		this(serverHost, ProtocolServer.SERVER_PORT);
	}

	/**
	 * This ProtocolClient sends the operations to serverHost and ProtocolServer.SERVER_PORT.
	 * @param serverHost
	 * @param port
	 * @throws UnknownHostException
	 */
	public ProtocolClient(String serverHost, int port) throws UnknownHostException {
		this(InetAddress.getByName(serverHost), port);
	}

	/**
	 * This ProtocolClient sends the operations to serverHost and ProtocolServer.SERVER_PORT.
	 * @param serverHost
	 * @throws UnknownHostException
	 */
	public ProtocolClient(String serverHost) throws UnknownHostException {
		this(serverHost, ProtocolServer.SERVER_PORT);
	}

	/**
	 * Sets the server host that this ProtocolClient sends operations to.
	 * @param  serverHost
	 * @uml.property  name="serverHost"
	 */
	public void setServerHost(InetAddress serverHost) {
		this.serverHost = serverHost;
	}
	
	private ByteArrayOutputStream packetStream = new ByteArrayOutputStream(1024);
	
	private void writeUnsignedByte(short b) {
		if (b >= 0xFF) {
			throw new IllegalArgumentException("Cannot write " + b + " as unsigned byte");
		}
		packetStream.write(b & 0x00FF);
	}
	private void writeUnsignedShort(int s) {
		packetStream.write((short)((s >> 8) & 0x00FF));
		packetStream.write((short)((s >> 0) & 0x00FF));
	}
	private void writeUnsignedByteArray(short[] b, int offset, int length) {
		for (int i = 0; i < length && i + offset < b.length; i++) {
			packetStream.write(b[i + offset]);
		}
	}
	private void reset() {
		packetStream.reset();
	}

	private void logPacket(byte[] packet, InetAddress host) {
		StringBuilder buffer = new StringBuilder();
		buffer.append(host);
		buffer.append(":");
		for (int i = 0; i < packet.length; i++) {
			if (i > 0) {
				buffer.append(",");
			}
			byte b = packet[i];
			buffer.append(b < 10 ? "  " : b < 100 ? " " : "");
			buffer.append(b);
		}
		System.out.println(buffer);
	}
	
	private boolean logPacketsOn = false;
	
	/**
	 * Sets whether the operation packets are logged.
	 * @param logPacketsOn  whether the operation packets are logged.
	 * @uml.property  name="logPacketsOn"
	 */
	public void setLogPacketsOn(boolean logPacketsOn) {
		this.logPacketsOn = logPacketsOn;
	}

	/**
	 * @uml.property  name="sendPacketsOn"
	 */
	private boolean sendPacketsOn = true;
	
	/**
	 * Returns whether the operation packets are logged.
	 * @return  whether the operation packets are logged.
	 * @uml.property  name="sendPacketsOn"
	 */
	public boolean isSendPacketsOn() {
		return this.sendPacketsOn;
	}

	/**
	 * Returns whether the operation packets actually are sent.
	 * @param sendPacketsOn  whether the operation packets actually are sent.
	 * @uml.property  name="sendPacketsOn"
	 */
	public void setSendPacketsOn(boolean sendPacketsOn) {
		this.sendPacketsOn = sendPacketsOn;
	}
	
	private DatagramSocket datagramSocket = null;
	
	/**
	 * Sends the currently collected operations. 
	 * @throws IOException
	 */
	public void send() throws IOException {
		if (packetStream.size() <= 0) {
			throw new IllegalStateException("Cannot send packet with no operations");
		}
		InetAddress host = serverHost;
		byte[] packet = packetStream.toByteArray();
		if (logPacketsOn) {
			logPacket(packet, host);
		}
		if (sendPacketsOn) {
			IOException ioe = null;
			try {
				if (datagramSocket == null) {
					datagramSocket = new DatagramSocket();
				}
				DatagramPacket datagram = new DatagramPacket(packet, packet.length, host, serverPort);
				datagramSocket.send(datagram);
			} catch (SocketException e) {
				ioe = e;
				System.err.println("Datagram SocketException: " + e);
			} catch (IOException e) {
				ioe = e;
				System.err.println("Datagram IOException: " + e);
			}
			if (ioe != null) {
				throw ioe;
			}
		}
		reset();
	}

	public void clearWall() {
		writeUnsignedByte(CLEAR);
	}
	
	public void fillRect(short x, short y, short w, short h, short value) {
		writeUnsignedByte(FILL_RECT);
		writeUnsignedByte(x);
		writeUnsignedByte(y);
		writeUnsignedByte(w);
		writeUnsignedByte(h);
		writeUnsignedByte(value);
	}

	public void getLeds(short x, short y, short w, short h, short[] ip, int port) {
		writeUnsignedByte(GET_LEDS);
		writeUnsignedByte(x);
		writeUnsignedByte(y);
		writeUnsignedByte(w);
		writeUnsignedByte(h);
		writeUnsignedByteArray(ip, 0, ip.length);
		writeUnsignedShort(port);
	}

	public void nop() {
		writeUnsignedByte(NOP);
	}

	public void setLed(short x, short y, short value) {
		writeUnsignedByte(SET_LED);
		writeUnsignedByte(x);
		writeUnsignedByte(y);
		writeUnsignedByte(value);
	}

	public void setLeds(short x, short y, short w, short h, short[] value, int offset) {
		writeUnsignedByte(SET_LEDS);
		writeUnsignedByte(x);
		writeUnsignedByte(y);
		writeUnsignedByte(w);
		writeUnsignedByte(h);
		writeUnsignedByteArray(value, offset, w * h);
	}

	public void sync() {
		writeUnsignedByte(SYNC);
	}
}
