/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.logging.Level;
import java.util.logging.Logger;

import no.ntnu.idi.wall.model.DefaultWallModel;
import no.ntnu.idi.wall.model.WallModel;
import no.ntnu.idi.wall.ui.WallPanel;

/**
 * Implementation of a server, which receives packets containing a sequence of serialized operations. These operations are then decoded and executed on/by a provided Protocol implementation. 
 * @author  Hallvard Trtteberg
 */
public class ProtocolServer implements Runnable {

    private static final Logger log = Logger.getLogger(ProtocolServer.class.getName());

	/**
	 * @uml.property  name="protocol"
	 * @uml.associationEnd  
	 */
	private Protocol protocol;
	private int serverPort;
	
	/**
	 * The default server port
	 */
	public final static int SERVER_PORT = 5001;

	/**
	 * This instance will bind to the provided server port and
	 * execute operations on the given Protocol implementation
	 * @param serverPort the port to bind to
	 * @param protocol the Protocol implementation to execute operations on
	 */
	public ProtocolServer(int serverPort, Protocol protocol) {
		this.serverPort = serverPort;
		this.protocol = protocol;
	}
	
	/**
	 * This instance will bind to the default server port and
	 * execute operations on the given Protocol implementation
	 * @param protocol the Protocol implementation to execute operations on
	 */
	public ProtocolServer(Protocol protocol) {
		this(SERVER_PORT, protocol);
	}
	
	/**
	 * This instance will bind to the default server port and
	 * execute operations on a fresh ProtocolModel instance.
	 */
	public ProtocolServer() {
		this(new ProtocolModel());
	}

	private DatagramSocket datagramSocket = null;

	private byte[] packet = new byte[2 << 12];

	/**
	 * Implementation of Runnable that listens for DatagramPackets on the server port
	 */
	public void run() {
		if (datagramSocket == null) {
			try {
				datagramSocket = new DatagramSocket(serverPort);
			} catch (SocketException e) {
				System.err.println(e);
			}
		}
		while (true) {
			DatagramPacket packet = new DatagramPacket(this.packet, this.packet.length);
			try {
				datagramSocket.receive(packet);
				log.log(Level.INFO, "Received packet from " + packet.getAddress() + " of length " + packet.getLength());
				receive(packet.getData(), packet.getOffset(), packet.getLength());
			} catch (IOException e) {
				System.err.println(e);
				break;
			}
		}
	}
	
	private short[] buffer = null;
	
	private void receive(byte[] packet, int offset, int length) {
//		System.out.println("Receiving " + length + " bytes from " + offset);
		if (buffer == null || buffer.length < length) {
			buffer = new short[length];
		}
		for (int i = 0; i < length; i++) {
			buffer[i] = (short)(packet[offset + i] & 0x00FF);
		}
		int pos = 0;
		while (pos < length) {
//			System.out.println("Op @ " + pos + " = " + buffer[pos]);
			pos = receive(buffer, pos);
		}
		protocol.sync();
	}

	private short getUnsignedByte(short[] packet, int offset) {
		return (short)(packet[offset] & 0x00FF);
	}
	private int getUnsignedShort(short[] packet, int offset) {
		int i1 = packet[offset] 	& 0x000F;
		int i2 = packet[offset + 1] & 0x000F;
		return (i1 << 8) | i2;
	}

	private int receive(short[] packet, int offset) {

		log.log(Level.INFO, "Received opcode " + Protocol.OPCODES[packet[offset]]);

		switch (packet[offset++]) {
		
		case Protocol.NOP: 		protocol.nop();   		break;
		case Protocol.SYNC: 	protocol.sync();		break;
		case Protocol.CLEAR: 	protocol.clearWall(); 	break;
		
		case Protocol.SET_LED: {
			short x = getUnsignedByte(packet, offset++);
			short y = getUnsignedByte(packet, offset++);
			short value = getUnsignedByte(packet, offset++);
			protocol.setLed(x, y, value);
			break;
		}
		
		case Protocol.FILL_RECT: {
			short x = getUnsignedByte(packet, offset++);
			short y = getUnsignedByte(packet, offset++);
			short w = getUnsignedByte(packet, offset++);
			short h = getUnsignedByte(packet, offset++);
			short value = getUnsignedByte(packet, offset++);
			protocol.fillRect(x, y, w, h, value);
			break;
		}
		
		case Protocol.SET_LEDS: {
			short x = getUnsignedByte(packet, offset++);
			short y = getUnsignedByte(packet, offset++);
			short w = getUnsignedByte(packet, offset++);
			short h = getUnsignedByte(packet, offset++);
			protocol.setLeds(x, y, w, h, packet, offset);
			offset += w * h;
			break;
		}
		
		case Protocol.GET_LEDS: {
			short x = getUnsignedByte(packet, offset++);
			short y = getUnsignedByte(packet, offset++);
			short w = getUnsignedByte(packet, offset++);
			short h = getUnsignedByte(packet, offset++);
			short[] ip = new short[] {
					getUnsignedByte(packet, offset++),
					getUnsignedByte(packet, offset++),
					getUnsignedByte(packet, offset++),
					getUnsignedByte(packet, offset++),
			};
			int port = getUnsignedShort(packet, offset);
			offset += 2;
			protocol.getLeds(x, y, w, h, ip, port);
			break;
		}
		}
		return offset;
	}

	/**
	 * A main method that starts a server Thread.
	 * @param args not used
	 */
	public static void main(String[] args) {
		WallModel model = new DefaultWallModel();
		Protocol protocol = new ProtocolModel(model);
		ProtocolServer server = new ProtocolServer(protocol);
		Thread thread = new Thread(server);
		thread.start();
		WallPanel.createWallFrame(model);
	}
}
