/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.text.MessageFormat;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import no.ntnu.idi.wall.model.WallListener;
import no.ntnu.idi.wall.model.WallModel;

/**
 * This class updates the actual physical network-connected wall, by listening to changes to a WallModel and sending network packages to the panels accordingly. The model is typically updated by the ProtocolServer, when it receices packages from other clients. Thus, the ProtocolServer is the link to the rest of the internet, the WallNetworkConnector is the link to the wall, with the common WallModel sitting in between. The WallNetworkConnector is also a TimerTask, to periodically refresh the panels, to keep them awake
 * @author  Hallvard Trtteberg
 */
public class WallNetworkConnector extends TimerTask implements WallListener {

    private static final Logger log = Logger.getLogger(WallNetworkConnector.class.getName());

	private final static byte
		NO_OP = 0
		;

	private final static byte PACKET_DATA_IMMEDIATE = 1
//		PACKET_SET,
//		PACKET_DATA = 3,
//		PACKET_UPDATE = 4
;

	private byte[][][] packets = new byte[WallModel.WIDTH / WallModel.NODE_WIDTH][WallModel.HEIGHT / WallModel.NODE_HEIGHT][];
	
	private static int BOARD_PORT = 5001;
	
	private String inetAddressFormat = "10.0.10.{0}";

	private InetAddress[] hosts = new InetAddress[96];

	/**
	 * Gets the host address of the panel with the given number.
	 * The panels are number left to right, top down;
	 *   0 ... 15
	 *  16 ... 31
	 *  32 ... 47
	 *  48 ... 63
	 *  64 ... 77
	 *  78 ... 91
	 * @param n
	 * @return the host address of the panel with the given number
	 */
	public InetAddress getHost(int n) {
		if (hosts[n] == null) {
			try {
				hosts[n] = InetAddress.getByName(MessageFormat.format(inetAddressFormat, n + 1));
			} catch (UnknownHostException e) {
			}
		}
		return hosts[n];
	}

	private void logPacket(byte[] packet, InetAddress host) {
		StringBuilder buffer = new StringBuilder();
		buffer.append(host);
		buffer.append(":");
		for (int i = 1; i < packet.length; i++) {
			buffer.append((i-1) % WallModel.NODE_WIDTH == 0 ? "\n" : ",");
			byte b = packet[i];
			buffer.append(b < 10 ? "  " : b < 100 ? " " : "");
			buffer.append(b);
		}
        log.log(Level.INFO, buffer.toString());
	}
	
	private boolean logPacketsOn = true;

	/**
	 * Sets whether the prepared packets are logged.
	 * @param logPacketsOn  whether the prepared packets are logged
	 * @uml.property  name="logPacketsOn"
	 */
	public void setLogPacketsOn(boolean logPacketsOn) {
		this.logPacketsOn = logPacketsOn;
	}

	/**
	 * @uml.property  name="sendPacketsOn"
	 */
	private boolean sendPacketsOn = true;
	
	/**
	 * Tells whether the prepared packets are sent.
	 * @return  whether the prepared packets are sent.
	 * @uml.property  name="sendPacketsOn"
	 */
	public boolean isSendPacketsOn() {
		return this.sendPacketsOn;
	}

	/**
	 * Sets whether the prepared packets are sent.
	 * @param sendPacketsOn  whether the prepared packets are sent
	 * @uml.property  name="sendPacketsOn"
	 */
	public void setSendPacketsOn(boolean sendPacketsOn) {
		this.sendPacketsOn = sendPacketsOn;
	}
	
	private DatagramSocket datagramSocket = null;
	
	/**
	 * This method logs and sends the packets, as DatagramPackets, depending on the flags.
	 * @param packet the packet includes the PACKET_DATA_IMMEDIATE header byte and all the led values for the panel
	 * @param hostN the panel number
	 * @param sendPackets
	 * @param logPackets
	 */
	private void sendPacket(byte[] packet, int hostN, boolean sendPackets, boolean logPackets) {
		InetAddress host = getHost(hostN);
		if (logPackets) {
			logPacket(packet, host);
		}
		if (sendPackets) {
			try {
				if (datagramSocket == null) {
					datagramSocket = new DatagramSocket();
				}
				DatagramPacket datagram = new DatagramPacket(packet, packet.length, host, BOARD_PORT);
				datagramSocket.send(datagram);
			} catch (SocketException e) {
				System.err.println("Datagram SocketException: " + e);
			} catch (IOException e) {
				System.err.println("Datagram IOException: " + e);
			}
		}
	}
	
	/**
	 * This method contains the actual logic for mapping a WallModel to packets sent to the physical panels.
	 * The logic is as follows:
	 * - iterate across all panels that overlap with the rectangle that must be updated
	 * - if there is no packet data for this panel, meaning nothing has been sent to this panel yet, create it
	 * - iterate across all led values in the model, for this panel
	 * - get the led value and insert it into the packet at the appropriate position
	 * - operation header byte first, followed by rows of led values from the top
	 * - each row seems to be reversed, i.e. rightmost led values in row are first in the packet
	 * @param x the x-coordinate of the rectangle of the wall that must be updated
	 * @param y the y-coordinate of the rectangle of the wall that must be updated
	 * @param width the width of the rectangle of the wall that must be updated
	 * @param height the height of the rectangle of the wall that must be updated
	 * @param model the model containing the led values
	 */
	private void sendDataPackets(int x, int y, int width, int height, WallModel model) {
		if (packets == null) {
			packets = new byte[WallModel.WIDTH / WallModel.NODE_WIDTH][WallModel.HEIGHT / WallModel.NODE_HEIGHT][];
		}
		// iterate across all panels that overlap with the rectangle that must be updated
		for (int wx = x / WallModel.NODE_WIDTH; wx * WallModel.NODE_WIDTH < x + width; wx++) {
			for (int wy = y / WallModel.NODE_HEIGHT; wy * WallModel.NODE_HEIGHT < y + height; wy++) {
				byte[] packet = packets[wx][wy];
				// if there is no packet data for this panel, meaning nothing has been sent to this panel yet, create it
				if (packet == null) {
					packet = packets[wx][wy] = new byte[1 + WallModel.NODE_WIDTH * WallModel.NODE_HEIGHT];
					packet[0] = PACKET_DATA_IMMEDIATE;
				}
				// iterate across all led values in the model, for this panel
				for (int dx = 0; dx < WallModel.NODE_WIDTH; dx++) {
					for (int dy = 0; dy < WallModel.NODE_HEIGHT; dy++) {
						// get the led value and insert it into the packet at the appropriate position
						byte led = model.getLed(wx * WallModel.NODE_WIDTH + dx, wy * WallModel.NODE_HEIGHT + dy);
						// operation header byte first, followed by rows of led values from the top
						// each row seems to be reversed, i.e. rightmost led values in row are first in the packet
						int pos = 1 + dy * WallModel.NODE_WIDTH + (WallModel.NODE_WIDTH - dx - 1);
						if (led != packet[pos]) {
							packet[0] = PACKET_DATA_IMMEDIATE;
							packet[pos] = led;
						}
					}
				}
				sendPacket(packet, wx, wy, sendPacketsOn, logPacketsOn);
			}
		}
	}

	private void sendPacket(byte[] packet, int wx, int wy, boolean sendPackets, boolean logPackets) {
		if (packet[0] != NO_OP && sendPacketsOn) {
			sendPacket(packet, wy * WallModel.WIDTH / WallModel.NODE_WIDTH + wx, sendPackets, logPackets);
			packet[0] = NO_OP;
		}
	}

	public void ledChanged(int x, int y, byte value, WallModel model) {
	}

	public void ledsChanged(int x, int y, int width, int height, WallModel model) {
		if (x == 0 && y == 0 && width == WallModel.WIDTH && height == WallModel.HEIGHT) {
			packets = null;
		}
		sendDataPackets(x, y, width, height, model);
	}

	// from TimerTask, to refresh all panels periodically
	public void run() {
		for (int i = 0; i < packets.length; i++) {
			for (int j = 0; j < packets[i].length; j++) {
				byte[] packet = packets[i][j];
				if (packet != null && sendPacketsOn) {
					packet[0] = PACKET_DATA_IMMEDIATE;
					sendPacket(packet, i, j, sendPacketsOn, false);
				}
			}
		}
		log.log(Level.INFO, "Refreshed wall");
	}
}
