/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;

import no.ntnu.idi.wall.model.DefaultWallModel;
import no.ntnu.idi.wall.model.WallListener;
import no.ntnu.idi.wall.model.WallModel;

/**
 * @author  Simone
 */
public class WallPanel extends JPanel implements WallListener {

	/**
	 * @uml.property  name="model"
	 * @uml.associationEnd  
	 */
	private WallModel model;

	private boolean paintGrid = false;

	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (paintGrid) {
			paintGrid(g);
		}
		paintWall(g);
	}

	public int getLedX(int x) {
		return (x - getInsets().top) / getLedWidth();
	}

	public int getLedY(int y) {
		return (y - getInsets().left) / getLedHeight();
	}

	/**
	 * @uml.property  name="ledRenderer"
	 * @uml.associationEnd  
	 */
	private LedRenderer ledRenderer = new StandardLedRenderer();
	
	/**
	 * @param ledRenderer
	 * @uml.property  name="ledRenderer"
	 */
	public void setLedRenderer(LedRenderer ledRenderer) {
		this.ledRenderer = ledRenderer;
	}

	private void paintLed(int x, int y, int value, Graphics g, int ledX, int ledY, int w, int h) {
		ledRenderer.paintLed(x, y, value, g, ledX, ledY, w, h);
	}
	
	private Rectangle tempClip = new Rectangle();
	
	private void paintWall(Graphics g) {
		int gridSize = (paintGrid ? 1 : 0);
		g.getClipBounds(tempClip);
		Insets insets = getInsets();
		int cellWidth = getLedWidth(), cellHeight = getLedHeight();
		for (int x = 0; x < WallModel.WIDTH; x++) {
			for (int y = 0; y < WallModel.HEIGHT; y++) {
				int gx = insets.left + gridSize + x * cellWidth;
				int gy = insets.top + gridSize + y * cellHeight;
				int w = cellWidth - gridSize;
				int h = cellHeight - gridSize;
				if (tempClip.contains(gx        , gy) ||
					tempClip.contains(gx + w - 1, gy) ||
					tempClip.contains(gx        , gy + h - 1) ||
					tempClip.contains(gx + w - 1, gy + h - 1)) {
					paintLed(x, y, model.getLed(x, y), g, gx, gy, w, h);
				}
			}
		}
	}

	public int getLedWidth() {
		Insets insets = getInsets();
		return (getWidth() - insets.left - insets.right) / WallModel.WIDTH;
	}

	public int getLedHeight() {
		Insets insets = getInsets();
		return (getHeight() - insets.top - insets.bottom) / WallModel.HEIGHT;
	}

	private void repaintLed(int x, int y) {
		Insets insets = getInsets();
		int cellWidth = getLedWidth();
		int cellHeight = getLedHeight();
		int gridSize = (paintGrid ? 1 : 0);
		repaint(insets.left + gridSize + x * cellWidth, insets.top + gridSize + y * cellHeight,
				cellWidth - gridSize, cellHeight - gridSize);
	}

	private int dim(int n, int m) {
		return 255 - (255 - n) / m;
	}
	private Color dim(Color c, int m) {
		return new Color(dim(c.getRed(), m), dim(c.getGreen(), m), dim(c.getBlue(), m));
	}
	
	private void paintGrid(Graphics g) {
		Color foreground = getForeground();
		Color gridColor = dim(foreground, 4);
		g.setColor(gridColor);
		Insets insets = getInsets();
		int cellWidth = getLedWidth(), cellHeight = getLedHeight();
		for (int y = 0; y <= WallModel.HEIGHT; y++) {
			int cellY = insets.top + y * cellHeight;
			g.drawLine(insets.left, cellY, insets.left + WallModel.WIDTH * cellWidth, cellY);
		}
		for (int x = 0; x <= WallModel.WIDTH; x++) {
			int cellX = insets.left + x * cellWidth;
			g.drawLine(cellX, insets.top, cellX, insets.top + WallModel.HEIGHT * cellHeight);
		}
		g.setColor(foreground);
	}
	
	public WallPanel(final WallModel model) {
		setModel(model);
		model.addWallListener(this);
//		addMouseListener(toggleHandler);
	}
	
	private MouseAdapter toggleHandler = new MouseAdapter() {
		public void mousePressed(MouseEvent event) {
			int x = getLedX(event.getX()), y = getLedY(event.getY());
			model.setLed(x, y, (byte)(127 - model.getLed(x, y)));
		}
	};
	
	/**
	 * @param model
	 * @uml.property  name="model"
	 */
	public void setModel(WallModel model) {
		this.model = model;
	}
	
	public void ledChanged(int x, int y, byte value, WallModel model) {
		if (this.model == model) {
			repaintLed(x, y);
		}
	}

	public void ledsChanged(int x0, int y0, int dx, int dy, WallModel model) {
		if (this.model == model) {
			for (int x = x0; x < x0 + dx; x++) {
				for (int y = y0; y < y0 + dy; y++) {
					repaintLed(x, y);
				}
			}
		}
	}

	public static void createWallFrame(WallModel model) {
		JFrame frame = new JFrame("WallPanel");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		WallPanel wallPanel = new WallPanel(model);
		frame.setContentPane(wallPanel);
//		wallPanel.setSize(WallModel.WIDTH * 12, WallModel.HEIGHT * 12);
		frame.setSize(WallModel.WIDTH * 10 + 8, WallModel.HEIGHT * 10 + 34);
//		frame.pack();
		frame.setLocation(450, 400);
		frame.setVisible(true);
	}
	
	public static void main(String[] args) {
		createWallFrame(new DefaultWallModel());
	}
}
